use std::future;
use std::path::PathBuf;
use std::sync::Arc;

use log::info;
use ropey::Rope;
use serde::Deserialize;
use serde_json::Value;
use tower_lsp::jsonrpc::Result;
use tower_lsp::lsp_types::*;

use crate::core::fast_hdlparam::FastHdlparam;
use crate::core::sv_parser::make_fast_from_syntaxtree;

use crate::server::Backend;
use crate::sources::parse;

#[derive(Debug, Deserialize)]
pub struct CustomRequestParams {
    // 定义你需要的参数
    pub data: Value,
}


#[derive(Clone)]
pub struct CustomRequest;

impl <'a>tower_lsp::jsonrpc::Method<&'a Arc<Backend>, (), Result<i32>> for CustomRequest {
    type Future = future::Ready<Result<i32>>;

    fn invoke(&self, _server: &'a Arc<Backend>, _params: ()) -> Self::Future {
        future::ready(custom_request())
    }
}

pub fn custom_request() -> Result<i32> {
    Ok(123)
}

#[derive(Clone)]
pub struct DoFastApi;

impl <'a>tower_lsp::jsonrpc::Method<&'a Arc<Backend>, (TextDocumentItem, ), Result<FastHdlparam>> for DoFastApi {
    type Future = future::Ready<Result<FastHdlparam>>;

    fn invoke(&self, _server: &'a Arc<Backend>, _params: (TextDocumentItem, )) -> Self::Future {
        let doc = _params.0;
        future::ready(do_fast(doc))
    }
}

/// 前端交互接口： do_fast，输入文件路径，计算出对应的 fast 结构
pub fn do_fast(doc: TextDocumentItem) -> Result<FastHdlparam> {
    info!("lsp get doc: {:?}", doc);

    let uri = doc.uri;
    let text = Rope::from_str(&doc.text);
    // fast 解析不需要 include
    let includes: Vec<PathBuf> = Vec::new();
    
    let parse_result = parse(
        &text, 
        &uri, 
        &None,
        &includes
    );

    if let Some(syntax_tree) = parse_result {
        let hdlparam = make_fast_from_syntaxtree(syntax_tree);
        return Ok(hdlparam);
    }

    let api_error = tower_lsp::jsonrpc::Error {
        code: tower_lsp::jsonrpc::ErrorCode::ParseError,
        message: std::borrow::Cow::Owned("message".to_string()),
        data: None
    };

    Err(api_error)
}